<?php

namespace Drupal\file_compressor_field\Plugin\Field\FieldType;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;

/**
 * Represents a configurable entity file compressor field.
 */
class FileCompressorFieldItemList extends FileFieldItemList implements FileCompressorFieldItemListInterface {

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    $langcodes = ($this->getFieldDefinition()->isTranslatable()) ? [$this->getLangcode()] : array_keys(\Drupal::languageManager()->getLanguages());
    foreach (array_filter($this->getFieldDefinition()->getSetting('compressed_fields')) as $field_to_compress) {
      foreach ($langcodes as $current_langcode) {
        $entity = $this->getEntity();
        if (!$entity->hasTranslation($current_langcode)) {
          continue;
        }
        $field_items = $entity->getTranslation($current_langcode)->{$field_to_compress};
        if (!$field_items->isEmpty()) {
          foreach ($field_items as $field_item) {
            if (isset($field_item->target_id) && $field_item->target_id > 0 && !isset($files[$field_item->target_id])) {
              $current_file = File::load($field_item->target_id);
              $files[$current_file->fid->value] = $current_file->uri->value;
            }
          }
        }
      }
    }

    if (!empty($files) && $compressed_file = $this->generateCompressedFile($files)) {
      $item = [
        'target_id' => $compressed_file->fid->value,
        'display' => 1,
      ];
      $this->set(0, $item);
    }
  }

  /**
   * Generates a compressed file given a list of URIs.
   *
   * @param array $files
   *   Array containing file uris to compress,.
   *
   * @return bool|\Drupal\Core\Entity\EntityInterface
   *   The compressed ile entity, FALSE otherwise.
   */
  protected function generateCompressedFile($files) {
    $file_compressor_manager = \Drupal::service('plugin.manager.file_compressor');
    /** @var \Drupal\file\Entity\FileCompressorPluginInterface $file_compressor */
    $file_compressor = $file_compressor_manager->createInstance($this->getSetting('file_compressor'));
    $compressed_field_components = empty($this->target_id) ? FALSE : File::load($this->target_id);
    $create_compressed_file = $this->isEmpty() || $this->getEntity()->isNewRevision() || !$compressed_field_components;
    if ($create_compressed_file) {
      $directory_uri = $this->getUploadLocation();

      /** @var \Drupal\Core\File\FileSystemInterface $file_system */
      $file_system = \Drupal::service('file_system');

      if (!$file_system->prepareDirectory($directory_uri, FileSystemInterface::CREATE_DIRECTORY)) {
        \Drupal::messenger()->addMessage('The directory wasn\'t created or was not writable', 'error');
        return FALSE;
      }
      do {
        $file_uri = $file_compressor->generateCompressedFileUri($directory_uri . '/compressed_file_' . \Drupal::service('password_generator')->generate(4));
        $exists = file_exists($file_uri);
      } while ($exists);
    }
    else {
      if (!$compressed_field_components) {
        \Drupal::logger('file_compressor')->error('File not found: ' . $this->target_id);
        return FALSE;
      }

      $file_uri = $compressed_field_components->getFileUri();
    }

    if ($file_compressor->generateCompressedFile($file_uri, $files)) {
      if ($create_compressed_file) {
        $user = \Drupal::currentUser();
        /** @var \Drupal\file\FileStorageInterface $file_storage */
        $file_storage = \Drupal::entityTypeManager()->getStorage('file');
        $compressed_field_components = $file_storage->create([
          'uri' => $file_uri,
          'uid' => $user->id(),
          'status' => FileInterface::STATUS_PERMANENT,
        ]);
        drupal_register_shutdown_function([get_class($this), 'fileCompressorFieldStore'], $this, $compressed_field_components);
      }
      else {
        $compressed_field_components->setFileUri($file_uri);
      }
      $compressed_field_components->save();
      return $compressed_field_components;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getUploadLocation($data = []) {
    $settings = $this->getFieldDefinition()->getSettings();
    $destination = trim($settings['file_directory'], '/');

    // Replace tokens.
    $destination = \Drupal::token()->replace($destination, $data);

    return $settings['uri_scheme'] . '://' . $destination;
  }

  /**
   * Renames and stores the compressed file in its definitive destination.
   *
   * @param \Drupal\file_compressor_field\Plugin\Field\FieldType\FileCompressorFieldItemListInterface $field
   *   The FileCompressorFieldItemList definition.
   * @param \Drupal\file\Entity\File $file
   *   The File entity to move and store.
   */
  public static function fileCompressorFieldStore(FileCompressorFieldItemListInterface $field, File $file) {
    /** @var \Drupal\Core\File\FileSystemInterface $file_system */
    $file_system = \Drupal::service('file_system');

    $entity = \Drupal::service('entity.repository')->loadEntityByUuid($field->getFieldDefinition()->getTargetEntityTypeId(), $field->getEntity()->uuid->value);
    $file_compressor_manager = \Drupal::service('plugin.manager.file_compressor');
    /** @var \Drupal\file\Entity\FileCompressorPluginInterface $file_compressor */
    $file_compressor = $file_compressor_manager->createInstance($field->getSetting('file_compressor'));
    $values[] = $field->getEntity()->getEntityTypeId();
    $values[] = $entity->id();
    // Check whether the entity type supports revisions and initialize it if so.
    if ($field->getEntity()->getEntityType()->isRevisionable()) {
      $values[] = $entity->{$field->getEntity()->getEntityType()->getKey('revision')}->value;
    }
    if ($field->getFieldDefinition()->isTranslatable()) {
      $values[] = $field->getLangcode();
    }
    $destination = $field->getUploadLocation() . '/' . implode('-', $values);
    $destination = $file_compressor->generateCompressedFileUri($destination);
    if ($destination = self::fileUnmanagedMove($file->getFileUri(), $destination, FileSystemInterface::EXISTS_RENAME)) {
      $file->setFileUri($destination);
      $file->setFilename($file_system->basename($destination));
      $file->save();
    }
  }

  /**
   * Moves a file to a new location without database changes or hook invocation.
   */
  public static function fileUnmanagedMove($source, $destination = NULL, $replace = FileSystemInterface::EXISTS_RENAME) {
    try {
      $file_system = \Drupal::service('file_system');

      // Build a destination URI if necessary.
      if (!isset($destination)) {
        $uri = \Drupal::config('system.file')->get('default_scheme') . '://' . $file_system->basename($source);
        /** @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager */
        $stream_wrapper_manager = \Drupal::service('stream_wrapper_manager');
        $destination = $stream_wrapper_manager->normalizeUri($uri);
      }
      return $file_system->move($source, $destination, $replace);
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

}
